@@ -23,16 +23,19 @@
 #include <tds.h>
 
 #ifdef __cplusplus
-extern "C" {
-#endif 
+extern "C"
+{
+#if 0
+}
+#endif
+#endif
 
-static char  rcsid_cspublic_h [ ] =
-         "$Id: cspublic.h,v 1.28 2003/01/28 14:28:52 freddy77 Exp $";
-static void *no_unused_cspublic_h_warn[]={rcsid_cspublic_h, no_unused_cspublic_h_warn};
+static char rcsid_cspublic_h[] = "$Id: cspublic.h,v 1.47 2004/01/06 09:29:25 freddy77 Exp $";
+static void *no_unused_cspublic_h_warn[] = { rcsid_cspublic_h, no_unused_cspublic_h_warn };
 
-typedef int CS_RETCODE ;
+typedef int CS_RETCODE;
 
-#define CS_PUBLIC 
+#define CS_PUBLIC
 #define CS_STATIC static
 
 typedef TDS_INT CS_INT;
@@ -40,12 +43,15 @@ typedef TDS_SMALLINT CS_SMALLINT;
 typedef TDS_TINYINT CS_TINYINT;
 typedef TDS_CHAR CS_CHAR;
 typedef TDS_UCHAR CS_BYTE;
+/* XX defined */
 typedef TDS_NUMERIC CS_NUMERIC;
 typedef float CS_REAL;
 typedef double CS_FLOAT;
 typedef char CS_BOOL;
 typedef void CS_VOID;
+/* XX defined, error, int instead of short... */
 typedef TDS_VARBINARY CS_VARBINARY;
+/* XX defined */
 typedef TDS_NUMERIC CS_DECIMAL;
 typedef TDS_UCHAR CS_IMAGE;
 typedef TDS_UCHAR CS_TEXT;
@@ -59,37 +65,92 @@ typedef TDS_UCHAR CS_BIT;
 #define CS_FAIL	   TDS_FAIL
 #define CS_SUCCEED TDS_SUCCEED
 #define CS_SIZEOF(x) sizeof(x)
+#define CS_NOMSG   -99
 
 #define CS_LAYER(x)    (((x) >> 24) & 0xFF)
 #define CS_ORIGIN(x)   (((x) >> 16) & 0xFF)
 #define CS_SEVERITY(x) (((x) >>  8) & 0xFF)
 #define CS_NUMBER(x)   ((x) & 0xFF)
 
-typedef struct cs_config
+#define CS_OBJ_NAME 400		/* full name length */
+#define CS_TP_SIZE  16		/* text pointer */
+#define CS_TS_SIZE  8		/* length of timestamp */
+
+typedef struct _cs_config
 {
-    short cs_expose_formats;
+	short cs_expose_formats;
 } CS_CONFIG;
 
 /* forward declarations */
-typedef struct cs_context CS_CONTEXT;
+typedef struct _cs_context CS_CONTEXT;
 typedef struct cs_clientmsg CS_CLIENTMSG;
 typedef struct cs_connection CS_CONNECTION;
-typedef struct cs_servermsg CS_SERVERMSG;
-typedef CS_RETCODE (*CS_CSLIBMSG_FUNC)(CS_CONTEXT *, CS_CLIENTMSG *);
-typedef CS_RETCODE (*CS_CLIENTMSG_FUNC)(CS_CONTEXT *, CS_CONNECTION *, CS_CLIENTMSG *);
-typedef CS_RETCODE (*CS_SERVERMSG_FUNC)(CS_CONTEXT *, CS_CONNECTION *, CS_SERVERMSG *);
+typedef struct _cs_servermsg CS_SERVERMSG;
+typedef CS_RETCODE(*CS_CSLIBMSG_FUNC) (CS_CONTEXT *, CS_CLIENTMSG *);
+typedef CS_RETCODE(*CS_CLIENTMSG_FUNC) (CS_CONTEXT *, CS_CONNECTION *, CS_CLIENTMSG *);
+typedef CS_RETCODE(*CS_SERVERMSG_FUNC) (CS_CONTEXT *, CS_CONNECTION *, CS_SERVERMSG *);
 
-struct cs_context
+/* 	Formerly CSREMOTE_PROC_PARAM, this structure can be used in other
+	places, too. */
+
+typedef struct _CS_PARAM
 {
-	CS_INT date_convert_fmt;
-	CS_CSLIBMSG_FUNC _cslibmsg_cb;
-	CS_CLIENTMSG_FUNC _clientmsg_cb;
-	CS_SERVERMSG_FUNC _servermsg_cb;
-	TDSCONTEXT *tds_ctx;
-    CS_CONFIG config;
+	struct _CS_PARAM *next;
+	char *name;
+	int status;
+	int type;
+	CS_INT maxlen;
+	CS_INT *datalen;
+	CS_SMALLINT *ind;
+	CS_BYTE *value;
+	int param_by_value;
+} CS_PARAM;
+
+
+ /* Code added for RPC functionality - SUHA */
+ /* RPC Code changes starts here */
+
+typedef CS_PARAM CSREMOTE_PROC_PARAM;
+
+typedef struct _CSREMOTE_PROC
+{
+	char *name;
+	CS_SMALLINT options;
+	CSREMOTE_PROC_PARAM *param_list;
+} CSREMOTE_PROC;
+
+/* RPC Code changes ends here */
+
+#define _CS_ERRHAND_INLINE 1
+#define _CS_ERRHAND_CB     2
+
+/* Code changed for error handling */
+/* Code changes starts here - CT_DIAG - 01 */
+
+/* This structure is used in CT_DIAG */
+
+struct cs_diag_msg_client
+{
+	CS_CLIENTMSG *clientmsg;
+	struct cs_diag_msg_client *next;
 };
 
-typedef struct cs_locale {
+struct cs_diag_msg_svr
+{
+	CS_SERVERMSG *servermsg;
+	struct cs_diag_msg_svr *next;
+};
+
+/* Code changes ends here - CT_DIAG - 01 */
+
+struct cs_diag_msg
+{
+	CS_CLIENTMSG *msg;
+	struct cs_diag_msg *next;
+};
+
+typedef struct _cs_locale
+{
 	char *language;
 	char *charset;
 	char *time;
@@ -108,32 +169,72 @@ struct cs_connection
 	CS_LOCALE *locale;
 };
 
-typedef struct cs_command
+
+#define CS_IODATA          (CS_INT)1600
+
+/* XX defined */
+typedef struct _cs_iodesc
+{
+	CS_INT iotype;
+	CS_INT datatype;
+	CS_LOCALE *locale;
+	CS_INT usertype;
+	CS_INT total_txtlen;
+	CS_INT offset;
+	CS_BOOL log_on_update;
+	CS_CHAR name[CS_OBJ_NAME];
+	CS_INT namelen;
+	CS_BYTE timestamp[CS_TS_SIZE];
+	CS_INT timestamplen;
+	CS_BYTE textptr[CS_TP_SIZE];
+	CS_INT textptrlen;
+} CS_IODESC;
+
+
+/* Structure CS_COMMAND changed for RPC functionality -SUHA */
+/* Added CSREMOTE_PROC *rpc to CS_COMMAND structure */
+
+typedef struct _cs_command
 {
 	CS_CHAR *query;
-	int cmd_done;
+	CS_INT command_type;
 	CS_CONNECTION *con;
-	void *userdata;
-	int userdata_len;
 	short dynamic_cmd;
-	char  *dyn_id; 
-    int   row_prefetched;
-    int   curr_result_type;
+	char *dyn_id;
+	int row_prefetched;
+	int results_state;
+	int curr_result_type;
+	/* Array Binding Code changes start here */
+	int bind_count;
+	/* Array Binding Code changes end here */
+	int get_data_item;
+	int get_data_bytes_returned;
+	CS_IODESC *iodesc;
+	CS_INT send_data_started;
+	CSREMOTE_PROC *rpc;
+	CS_PARAM *input_params;
 } CS_COMMAND;
 
+/* values for cs_command.results_state */
+
+#define _CS_RES_INIT            0
+#define _CS_RES_RESULTSET_EMPTY 1
+#define _CS_RES_RESULTSET_ROWS  2
+#define _CS_RES_STATUS          3
+#define _CS_RES_CMD_DONE        4
+#define _CS_RES_CMD_SUCCEED     5
+
 #define CS_MAX_MSG 1024
 #define CS_MAX_NAME 132
 #define CS_MAX_SCALE 77
-#define CS_MAX_PREC 77  /* used by php */
-#define CS_OBJ_NAME 132 /* ? */
-#define CS_TP_SIZE  16  /* text pointer */
-#define CS_TS_SIZE  8   /* length of timestamp */
+#define CS_MAX_PREC 77		/* used by php */
 #define CS_SQLSTATE_SIZE 8
 
 
 #define CS_SRC_VALUE   -2562
 
-typedef struct cs_datafmt {
+typedef struct _cs_datafmt
+{
 	int datatype;
 	int format;
 	int maxlength;
@@ -147,77 +248,70 @@ typedef struct cs_datafmt {
 	int usertype;
 } CS_DATAFMT;
 
-typedef TDS_MONEY  CS_MONEY;
+/* XX defined */
+typedef TDS_MONEY CS_MONEY;
+/* XX defined */
 typedef TDS_MONEY4 CS_MONEY4;
 
+/* XX defined */
 typedef TDS_DATETIME CS_DATETIME;
 
+/* XX defined */
 typedef TDS_DATETIME4 CS_DATETIME4;
 
-typedef struct cs_daterec {
-	CS_INT datesecond;
-	CS_INT dateminute;
-	CS_INT datehour;
+typedef struct _cs_daterec
+{
+	CS_INT dateyear;
+	CS_INT datemonth;
 	CS_INT datedmonth;
 	CS_INT datedyear;
-	CS_INT datemonth;
-	CS_INT dateyear;
-	CS_INT dateweek;
 	CS_INT datedweek;
+	CS_INT datehour;
+	CS_INT dateminute;
+	CS_INT datesecond;
 	CS_INT datemsecond;
 	CS_INT datetzone;
 } CS_DATEREC;
 
 typedef TDS_INT CS_MSGNUM;
 
-struct cs_clientmsg {
+/* XX defined */
+struct cs_clientmsg
+{
 	CS_INT severity;
 	CS_MSGNUM msgnumber;
 	CS_CHAR msgstring[CS_MAX_MSG];
 	CS_INT msgstringlen;
 	CS_INT osnumber;
-	CS_CHAR osstring[CS_MAX_MSG]; 
+	CS_CHAR osstring[CS_MAX_MSG];
 	CS_INT osstringlen;
 	CS_INT status;
 	CS_BYTE sqlstate[CS_SQLSTATE_SIZE];
 	CS_INT sqlstatelen;
 };
 
-struct cs_servermsg {
-	int severity;
-	int msgnumber;
-	int state;
-	int line;
-	int svrnlen;
-	char svrname[CS_MAX_NAME];
-	int proclen;
-	char proc[CS_MAX_NAME];
-	char text[CS_MAX_MSG];
-	int status;
+/* XX defined */
+struct _cs_servermsg
+{
+	CS_MSGNUM msgnumber;
+	CS_INT state;
+	CS_INT severity;
+	CS_CHAR text[CS_MAX_MSG];
+	CS_INT textlen; /* XX added */
+	CS_CHAR svrname[CS_MAX_NAME];
+	CS_INT svrnlen;
+	CS_CHAR proc[CS_MAX_NAME];
+	CS_INT proclen;
+	CS_INT line;
+	CS_INT status;
+	CS_BYTE sqlstate[CS_SQLSTATE_SIZE]; /* XX added */
+	CS_INT sqlstatelen; /* XX added */
 };
 
 /* status bits for CS_SERVERMSG */
 #define CS_HASEED 0x01
 
-typedef struct cs_blkdesc {
-	int dummy;	
-} CS_BLKDESC;
-
-typedef struct cs_iodesc {
-	CS_INT	iotype;
-	CS_INT	datatype;
-	CS_LOCALE	*locale;
-	CS_INT	usertype;
-	CS_INT	total_txtlen;
-	CS_INT	offset;
-	CS_BOOL	log_on_update;
-	CS_CHAR	name[CS_OBJ_NAME];
-	CS_INT	namelen;
-	CS_BYTE textptr[CS_TP_SIZE];
-	CS_INT	textptrlen;
-	CS_BYTE timestamp[CS_TS_SIZE];
-	CS_INT	timestamplen;
-} CS_IODESC;
+typedef struct _cs_blkdesc CS_BLKDESC;
 
 /* CS_CAP_REQUEST values */
 #define CS_REQ_LANG	1
@@ -312,7 +406,8 @@ typedef struct cs_iodesc {
 #define CS_DATA_NOINT8	35
 
 /* Properties */
-enum {
+enum
+{
 	CS_USERNAME = 1,
 /* These defines looks weird but programs can test support for defines, 
    compiler can check enum and there are no define side effecs */
@@ -347,25 +442,32 @@ enum {
 #define CS_CHARSETCNV CS_CHARSETCNV
 	CS_ANSI_BINDS,
 #define CS_ANSI_BINDS CS_ANSI_BINDS
-	CS_VER_STRING
+	CS_VER_STRING,
 #define CS_VER_STRING CS_VER_STRING
+	CS_SERVERNAME,
+#define CS_SERVERNAME CS_SERVERNAME
+	CS_PORT
+#define CS_PORT CS_PORT
 };
 
 /* Arbitrary precision math operators */
-enum {
+enum
+{
 	CS_ADD = 1,
 	CS_SUB,
 	CS_MULT,
 	CS_DIV
 };
 
-enum {
+enum
+{
 	CS_TDS_40 = 1,
 	CS_TDS_42,
 	CS_TDS_46,
 	CS_TDS_495,
 	CS_TDS_50,
-	CS_TDS_70
+	CS_TDS_70,
+	CS_TDS_80
 };
 
 /* bit mask values used by CS_DATAFMT.status */
@@ -387,6 +489,25 @@ enum {
 #define CS_CONSTAT_CONNECTED	0x01
 #define CS_CONSTAT_DEAD 	0x02
 
+/* Code added for CURSOR support */
+/* types accepted by ct_cursor */
+
+#define CS_CURSOR_DECLARE  1
+#define CS_CURSOR_OPEN     2
+#define CS_CURSOR_ROWS     3
+#define CS_CURSOR_OPTION   4
+#define CS_CURSOR_UPDATE   5
+#define CS_CURSOR_DELETE   6
+#define CS_CURSOR_DEALLOC  7
+#define CS_IMPLICIT_CURSOR 8
+#define CS_CURSOR_CLOSE    9
+
+/* internal defines for cursor processing */
+
+#define _CS_CURS_TYPE_UNACTIONED 0
+#define _CS_CURS_TYPE_REQUESTED  1
+#define _CS_CURS_TYPE_SENT       2
+
 /* options accepted by ct_options() */
 #define CS_OPT_ANSINULL		1
 #define CS_OPT_ANSIPERM		2
@@ -421,22 +542,24 @@ enum {
 #define CS_OPT_TRUNCIGNORE	31
 
 /* options accepted by ct_command() */
-enum ct_command_options {
-	CS_MORE, 
-	CS_END, 
-	CS_UNUSED, 
-	CS_RECOMPILE, 
-	CS_NO_RECOMPILE, 
-	CS_COLUMN_DATA, 
-	CS_BULK_DATA, 
-	CS_BULK_INIT, 
+enum ct_command_options
+{
+	CS_MORE,
+	CS_END,
+	CS_UNUSED,
+	CS_RECOMPILE,
+	CS_NO_RECOMPILE,
+	CS_COLUMN_DATA,
+	CS_BULK_DATA,
+	CS_BULK_INIT,
 	CS_BULK_CONT
 };
 
 
 /* bind formats, should be mapped to TDS types 
  * can be a combination of bit */
-enum {
+enum
+{
 	CS_FMT_UNUSED = 0,
 #define CS_FMT_UNUSED CS_FMT_UNUSED
 	CS_FMT_NULLTERM = 1,
@@ -471,6 +594,7 @@ enum {
 #define CS_LANG_CMD	7
 #define CS_ROW_FAIL	9
 #define CS_END_DATA	10
+#define CS_END_ITEM 11
 #define CS_CMD_SUCCEED	TDS_CMD_SUCCEED
 #define CS_CMD_FAIL	TDS_CMD_FAIL
 #define CS_CMD_DONE	TDS_CMD_DONE
@@ -501,7 +625,7 @@ enum {
 #define CS_COMP_BYLIST	54
 #define CS_COMP_OP	55
 #define CS_COMP_COLID	56
-#define CS_NO_COUNT	57
+#define CS_NO_COUNT	TDS_NO_COUNT
 #define CS_ROW_COUNT	59
 #define CS_OP_SUM	60
 #define CS_OP_AVG	61
@@ -548,8 +672,11 @@ enum {
 #define CS_SEND_DATA_CMD	119
 #define CS_SUPPORTED 120
 #define CS_EXPOSE_FMTS 121
+#define CS_DYNAMIC_CMD 122
+#define CS_MSGTYPE 123
 #define CS_VERSION	9114
 #define CS_EXTRA_INF	9121
+#define CS_CUR_CMD 133
 
 /* result_types */
 #define CS_COMPUTE_RESULT	TDS_COMPUTE_RESULT
@@ -590,9 +717,11 @@ enum {
 #define CS_VOID_TYPE	25
 #define CS_USHORT_TYPE	26
 #define CS_UNIQUE_TYPE	27
+#define CS_UNICHAR_TYPE  28
 
 /* cs_dt_info type values */
-enum {
+enum
+{
 	CS_MONTH = 1,
 #define CS_MONTH CS_MONTH
 	CS_SHORTMONTH,
@@ -608,7 +737,8 @@ enum {
 };
 
 /* DT_CONVFMT types */
-enum {
+enum
+{
 	CS_DATES_HMS = 1,
 #define CS_DATES_HMS CS_DATES_HMS
 	CS_DATES_SHORT,
@@ -663,9 +793,11 @@ enum {
 #define CS_DATES_YMD3_YYYY CS_DATES_YMD3_YYYY
 };
 
-typedef CS_RETCODE (*CS_CONV_FUNC)(CS_CONTEXT *context, CS_DATAFMT *srcfmt, CS_VOID *src, CS_DATAFMT *detsfmt, CS_VOID *dest, CS_INT *destlen);
+typedef CS_RETCODE(*CS_CONV_FUNC) (CS_CONTEXT * context, CS_DATAFMT * srcfmt, CS_VOID * src, CS_DATAFMT * detsfmt, CS_VOID * dest,
+				   CS_INT * destlen);
 
-typedef struct _cs_objname {
+typedef struct _cs_objname
+{
 	CS_BOOL thinkexists;
 	CS_INT object_type;
 	CS_CHAR last_name[CS_MAX_NAME];
@@ -678,7 +810,8 @@ typedef struct _cs_objname {
 	CS_INT threadlen;
 } CS_OBJNAME;
 
-typedef struct _cs_objdata {
+typedef struct _cs_objdata
+{
 	CS_BOOL actuallyexists;
 	CS_CONNECTION *connection;
 	CS_COMMAND *command;
@@ -687,16 +820,18 @@ typedef struct _cs_objdata {
 } CS_OBJDATA;
 
 /* Eventually, these should be in terms of TDS values */
-enum {
-        CS_OPT_SUNDAY,
-        CS_OPT_MONDAY,
-        CS_OPT_TUESDAY,
-        CS_OPT_WEDNESDAY,
-        CS_OPT_THURSDAY,
-        CS_OPT_FRIDAY,
-        CS_OPT_SATURDAY
+enum
+{
+	CS_OPT_SUNDAY,
+	CS_OPT_MONDAY,
+	CS_OPT_TUESDAY,
+	CS_OPT_WEDNESDAY,
+	CS_OPT_THURSDAY,
+	CS_OPT_FRIDAY,
+	CS_OPT_SATURDAY
 };
-enum {
+enum
+{
 	CS_OPT_FMTMDY,
 	CS_OPT_FMTDMY,
 	CS_OPT_FMTYMD,
@@ -704,7 +839,8 @@ enum {
 	CS_OPT_FMTMYD,
 	CS_OPT_FMTDYM
 };
-enum {
+enum
+{
 	CS_OPT_LEVEL0,
 	CS_OPT_LEVEL1,
 	CS_OPT_LEVEL3
@@ -715,7 +851,7 @@ enum {
 #define CS_TRUE	1
 
 #define SRV_PROC	CS_VOID
-#define CS_BLK_ROW	CS_VOID
+typedef struct _cs_blk_row CS_BLK_ROW;
 
 /* constants required for ct_diag (not jet implemented) */
 #define CS_INIT 36
@@ -725,33 +861,41 @@ enum {
 #define CS_SERVERMSG_TYPE 4701
 #define CS_ALLMSG_TYPE 4702
 
-CS_RETCODE cs_convert(CS_CONTEXT *ctx, CS_DATAFMT *srcfmt, CS_VOID *srcdata, CS_DATAFMT *destfmt, CS_VOID *destdata, CS_INT *resultlen);
-CS_RETCODE cs_ctx_alloc(CS_INT version, CS_CONTEXT **ctx);
-CS_RETCODE cs_ctx_global(CS_INT version, CS_CONTEXT **ctx);
-CS_RETCODE cs_ctx_drop(CS_CONTEXT *ctx);
-CS_RETCODE cs_config(CS_CONTEXT *ctx, CS_INT action, CS_INT property, CS_VOID *buffer, CS_INT buflen, CS_INT *outlen);
-CS_RETCODE cs_strbuild(CS_CONTEXT *ctx, CS_CHAR *buffer, CS_INT buflen, CS_INT *resultlen, CS_CHAR *text, CS_INT textlen, CS_CHAR *formats, CS_INT formatlen, ...);
-CS_RETCODE cs_dt_crack(CS_CONTEXT *ctx, CS_INT datetype, CS_VOID *dateval, CS_DATEREC *daterec);
-CS_RETCODE cs_loc_alloc(CS_CONTEXT *ctx, CS_LOCALE **locptr);
-CS_RETCODE cs_loc_drop(CS_CONTEXT *ctx, CS_LOCALE *locale);
-CS_RETCODE cs_locale(CS_CONTEXT *ctx, CS_INT action, CS_LOCALE *locale, CS_INT type, CS_VOID *buffer, CS_INT buflen, CS_INT *outlen);
-CS_RETCODE cs_dt_info(CS_CONTEXT *ctx, CS_INT action, CS_LOCALE *locale, CS_INT type, CS_INT item, CS_VOID *buffer, CS_INT buflen, CS_INT *outlen);
+CS_RETCODE cs_convert(CS_CONTEXT * ctx, CS_DATAFMT * srcfmt, CS_VOID * srcdata, CS_DATAFMT * destfmt, CS_VOID * destdata,
+		      CS_INT * resultlen);
+CS_RETCODE cs_ctx_alloc(CS_INT version, CS_CONTEXT ** ctx);
+CS_RETCODE cs_ctx_global(CS_INT version, CS_CONTEXT ** ctx);
+CS_RETCODE cs_ctx_drop(CS_CONTEXT * ctx);
+CS_RETCODE cs_config(CS_CONTEXT * ctx, CS_INT action, CS_INT property, CS_VOID * buffer, CS_INT buflen, CS_INT * outlen);
+CS_RETCODE cs_strbuild(CS_CONTEXT * ctx, CS_CHAR * buffer, CS_INT buflen, CS_INT * resultlen, CS_CHAR * text, CS_INT textlen,
+		       CS_CHAR * formats, CS_INT formatlen, ...);
+CS_RETCODE cs_dt_crack(CS_CONTEXT * ctx, CS_INT datetype, CS_VOID * dateval, CS_DATEREC * daterec);
+CS_RETCODE cs_loc_alloc(CS_CONTEXT * ctx, CS_LOCALE ** locptr);
+CS_RETCODE cs_loc_drop(CS_CONTEXT * ctx, CS_LOCALE * locale);
+CS_RETCODE cs_locale(CS_CONTEXT * ctx, CS_INT action, CS_LOCALE * locale, CS_INT type, CS_VOID * buffer, CS_INT buflen,
+		     CS_INT * outlen);
+CS_RETCODE cs_dt_info(CS_CONTEXT * ctx, CS_INT action, CS_LOCALE * locale, CS_INT type, CS_INT item, CS_VOID * buffer,
+		      CS_INT buflen, CS_INT * outlen);
 
-CS_RETCODE cs_calc(CS_CONTEXT *ctx, CS_INT op, CS_INT datatype, CS_VOID *var1, CS_VOID *var2, CS_VOID *dest);
-CS_RETCODE cs_cmp(CS_CONTEXT *ctx, CS_INT datatype, CS_VOID *var1, CS_VOID *var2, CS_INT *result);
-CS_RETCODE cs_conv_mult(CS_CONTEXT *ctx, CS_LOCALE *srcloc, CS_LOCALE *destloc, CS_INT *conv_multiplier);
-CS_RETCODE cs_diag(CS_CONTEXT *ctx, CS_INT operation, CS_INT type, CS_INT idx, CS_VOID *buffer);
-CS_RETCODE cs_manage_convert(CS_CONTEXT *ctx, CS_INT action, CS_INT srctype, CS_CHAR *srcname, CS_INT srcnamelen, CS_INT desttype, CS_CHAR *destname, CS_INT destnamelen, CS_INT *conv_multiplier, CS_CONV_FUNC *func);
-CS_RETCODE cs_objects(CS_CONTEXT *ctx, CS_INT action, CS_OBJNAME *objname, CS_OBJDATA *objdata);
-CS_RETCODE cs_set_convert(CS_CONTEXT *ctx, CS_INT action, CS_INT srctype, CS_INT desttype, CS_CONV_FUNC *func);
-CS_RETCODE cs_setnull(CS_CONTEXT *ctx, CS_DATAFMT *datafmt, CS_VOID *buffer, CS_INT buflen);
-CS_RETCODE cs_strcmp(CS_CONTEXT *ctx, CS_LOCALE *locale, CS_INT type, CS_CHAR *str1, CS_INT len1, CS_CHAR *str2, CS_INT len2, CS_INT *result);
-CS_RETCODE cs_time(CS_CONTEXT *ctx, CS_LOCALE *locale, CS_VOID *buffer, CS_INT buflen, CS_INT *outlen, CS_DATEREC *daterec);
-CS_RETCODE cs_will_convert(CS_CONTEXT *ctx, CS_INT srctype, CS_INT desttype, CS_BOOL *result);
+CS_RETCODE cs_calc(CS_CONTEXT * ctx, CS_INT op, CS_INT datatype, CS_VOID * var1, CS_VOID * var2, CS_VOID * dest);
+CS_RETCODE cs_cmp(CS_CONTEXT * ctx, CS_INT datatype, CS_VOID * var1, CS_VOID * var2, CS_INT * result);
+CS_RETCODE cs_conv_mult(CS_CONTEXT * ctx, CS_LOCALE * srcloc, CS_LOCALE * destloc, CS_INT * conv_multiplier);
+CS_RETCODE cs_diag(CS_CONTEXT * ctx, CS_INT operation, CS_INT type, CS_INT idx, CS_VOID * buffer);
+CS_RETCODE cs_manage_convert(CS_CONTEXT * ctx, CS_INT action, CS_INT srctype, CS_CHAR * srcname, CS_INT srcnamelen, CS_INT desttype,
+			     CS_CHAR * destname, CS_INT destnamelen, CS_INT * conv_multiplier, CS_CONV_FUNC * func);
+CS_RETCODE cs_objects(CS_CONTEXT * ctx, CS_INT action, CS_OBJNAME * objname, CS_OBJDATA * objdata);
+CS_RETCODE cs_set_convert(CS_CONTEXT * ctx, CS_INT action, CS_INT srctype, CS_INT desttype, CS_CONV_FUNC * func);
+CS_RETCODE cs_setnull(CS_CONTEXT * ctx, CS_DATAFMT * datafmt, CS_VOID * buffer, CS_INT buflen);
+CS_RETCODE cs_strcmp(CS_CONTEXT * ctx, CS_LOCALE * locale, CS_INT type, CS_CHAR * str1, CS_INT len1, CS_CHAR * str2, CS_INT len2,
+		     CS_INT * result);
+CS_RETCODE cs_time(CS_CONTEXT * ctx, CS_LOCALE * locale, CS_VOID * buffer, CS_INT buflen, CS_INT * outlen, CS_DATEREC * daterec);
+CS_RETCODE cs_will_convert(CS_CONTEXT * ctx, CS_INT srctype, CS_INT desttype, CS_BOOL * result);
 
 #ifdef __cplusplus
+#if 0
+{
+#endif
 }
-#endif 
-
 #endif
 
+#endif
